<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\QuarantineRequestForm;
use Modules\Qc\Entities\QuarantineRequestFormAttachment;

class QuarantineRequestFormController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('quarantine_request_form'), $filename);

            $quarantine_request_form_attachment = new QuarantineRequestFormAttachment();
            $quarantine_request_form_attachment->qr_form_id = 0;
            $quarantine_request_form_attachment->path = $filename;
            $quarantine_request_form_attachment->token = $token;
            $quarantine_request_form_attachment->save();

            $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allQuarantineRequestForms()
    {
        try {
            $query = "SELECT
            qc_quarantine_request_forms.*,
            qc_quarantine_request_form_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_quarantine_request_form_attachments
            INNER JOIN
            qc_quarantine_request_forms
            ON
                qc_quarantine_request_form_attachments.qr_form_id = qc_quarantine_request_forms.qr_form_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_quarantine_request_forms.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                pp_customer_order_plans_data.product_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_quarantine_request_forms.factory = hr_employee_sites.site_id
                GROUP BY
            qc_quarantine_request_forms.qr_form_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "qr_form_id" => $record->qr_form_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "allQuarantineRequestForms", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveQuarantineRequestForm(Request $request)
    {
        try {
            $qrf = new QuarantineRequestForm();
            $qrf->user_id = Auth::user()->id;
            $qrf->reference_no = $request->get('reference_no');
            $qrf->date = $request->get('date');
            $qrf->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');          
            $qrf->factory = $request->get('factory');
            $qrf->description = $request->get('description');
            if ($qrf->save()) {
                $token = $request->get('attachment_token');
                $attachment = QuarantineRequestFormAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->qr_form_id = $qrf->qr_form_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "saveQuarantineRequestForm", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $qrf =  QuarantineRequestForm::find($id);
            $qrf->user_id = Auth::user()->id;
            $qrf->reference_no = $request->get('reference_no');
            $qrf->date = $request->get('date');
            $qrf->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $qrf->factory = $request->get('factory');
            $qrf->description = $request->get('description');
            if ($qrf->update()) {
                $token = $request->get('attachment_token');
                $attachments = QuarantineRequestFormAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = QuarantineRequestFormAttachment::where('token', '=', $token)->first();
                        $attachment->qr_form_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = QuarantineRequestFormAttachment::where('qr_form_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = QuarantineRequestForm::where('qr_form_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = QuarantineRequestFormAttachment::where('qr_form_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getQrData($id)
    {
        try {
            $query = "SELECT
            qc_quarantine_request_forms.*,
            qc_quarantine_request_form_attachments.token
        FROM
            qc_quarantine_request_form_attachments
            INNER JOIN
            qc_quarantine_request_forms
            ON
                qc_quarantine_request_form_attachments.qr_form_id = qc_quarantine_request_forms.qr_form_id
        WHERE
            qc_quarantine_request_forms.qr_form_id = {$id}";

            $qrf =  DB::select($query);

            $responseBody = $this->responseBody(true, "QuarantineRequestFormController", "getQrData", $qrf[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "QuarantineRequestFormController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
