<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\RequestMethylBromide;
use Modules\Qc\Entities\RequestMethylBromideAttachment;

class RequestMethylBromideController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('request_mb'), $filename);

            $request_mb_attachment = new RequestMethylBromideAttachment();
            $request_mb_attachment->request_mb_id = 0;
            $request_mb_attachment->path = $filename;
            $request_mb_attachment->token = $token;
            $request_mb_attachment->save();

            $responseBody = $this->responseBody(true, "RequestMethylBromideController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_request_methyl_bromide.*,
            qc_request_methyl_bromide_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_request_methyl_bromide_attachments
            INNER JOIN
            qc_request_methyl_bromide
            ON
                qc_request_methyl_bromide_attachments.request_mb_id = qc_request_methyl_bromide.request_mb_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_request_methyl_bromide.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                pp_customer_order_plans_data.product_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_request_methyl_bromide.factory = hr_employee_sites.site_id
                GROUP BY
            qc_request_methyl_bromide.request_mb_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "request_mb_id" => $record->request_mb_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "RequestMethylBromideController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $mb = new RequestMethylBromide();
            $mb->user_id = Auth::user()->id;
            $mb->reference_no = $request->get('reference_no');
            $mb->date = $request->get('date');
            $mb->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $mb->factory = $request->get('factory');
            $mb->description = $request->get('description');
            if ($mb->save()) {
                $token = $request->get('attachment_token');
                $attachment = RequestMethylBromideAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->request_mb_id = $mb->request_mb_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "RequestMethylBromideController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $mb = RequestMethylBromide::find($id);
            $mb->user_id = Auth::user()->id;
            $mb->reference_no = $request->get('reference_no');
            $mb->date = $request->get('date');
            $mb->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $mb->factory = $request->get('factory');
            $mb->description = $request->get('description');
            if ($mb->update()) {
                $token = $request->get('attachment_token');
                $attachments = RequestMethylBromideAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = RequestMethylBromideAttachment::where('token', '=', $token)->first();
                        $attachment->request_mb_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "RequestMethylBromideController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = RequestMethylBromideAttachment::where('request_mb_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = RequestMethylBromide::where('request_mb_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = RequestMethylBromideAttachment::where('request_mb_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "RequestMethylBromideController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_request_methyl_bromide.*,
            qc_request_methyl_bromide_attachments.token
        FROM
            qc_request_methyl_bromide_attachments
            INNER JOIN
            qc_request_methyl_bromide
            ON
                qc_request_methyl_bromide_attachments.request_mb_id = qc_request_methyl_bromide.request_mb_id
        WHERE
            qc_request_methyl_bromide.request_mb_id = {$id}";

            $mb =  DB::select($query);

            $responseBody = $this->responseBody(true, "RequestMethylBromideController", "getData", $mb[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "RequestMethylBromideController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
